import unittest

from pyrage import ssh, RecipientError

from .utils import ssh_keypair


class TestIdentity(unittest.TestCase):
    def test_from_buffer(self):
        for filename in ["ed25519", "rsa4096", "rsa2048"]:
            _pubkey, privkey = ssh_keypair(filename)
            identity = ssh.Identity.from_buffer(privkey.encode())
            self.assertIsInstance(identity, ssh.Identity)


class TestRecipient(unittest.TestCase):
    def test_from_str(self):
        for filename in ["ed25519", "rsa4096", "rsa2048"]:
            pubkey, _privkey = ssh_keypair(filename)
            recipient = ssh.Recipient.from_str(pubkey)
            self.assertIsInstance(recipient, ssh.Recipient)

    def test_from_str_invalid(self):
        with self.assertRaisesRegex(RecipientError, "invalid SSH recipient"):
            ssh.Recipient.from_str("invalid ssh pubkey")



from pathlib import Path

_HERE = Path(__file__).parent
_ASSETS = _HERE / "assets"

assert _ASSETS.is_dir(), "missing test assets directory"


def ssh_keypair(name):
    (pub, priv) = (_ASSETS / f"{name}.pub", _ASSETS / name)
    return (pub.read_text(), priv.read_text())



import unittest

from pyrage import passphrase


class TestPassphrase(unittest.TestCase):
    def test_roundtrip(self):
        plaintext = b"junk"
        encrypted = passphrase.encrypt(plaintext, "some password")
        decrypted = passphrase.decrypt(encrypted, "some password")

        self.assertEqual(plaintext, decrypted)



from __future__ import annotations


class Identity:
    @classmethod
    def generate(cls) -> Identity:
        ...

    @classmethod
    def from_str(cls, v: str) -> Identity:
        ...

    def to_public(self) -> Recipient:
        ...


class Recipient:
    @classmethod
    def from_str(cls, v: str) -> Recipient:
        ...



from __future__ import annotations


class Identity:
    @classmethod
    def from_buffer(cls, buf: bytes) -> Identity:
        ...


class Recipient:
    @classmethod
    def from_str(cls, v: str) -> Recipient:
        ...



from __future__ import annotations
from typing import Sequence, Self, Optional, Protocol


class Callbacks(Protocol):
    def display_message(self, message: str) -> None:
        ...

    def confirm(self, message: str, yes_string: str, no_string: Optional[str]) -> Optional[bool]:
        ...

    def request_public_string(self, description: str) -> Optional[str]:
        ...

    def request_passphrase(self, description: str) -> Optional[str]:
        ...


class Recipient:
    @classmethod
    def from_str(cls, v: str) -> Recipient:
        ...

    def plugin(self) -> str:
        ...


class RecipientPluginV1:
    def __new__(cls, plugin_name: str, recipients: Sequence[Recipient], identities: Sequence[Identity], callbacks: Callbacks) -> Self:
        ...


class Identity:
    @classmethod
    def from_str(cls, v: str) -> Identity:
        ...

    @classmethod
    def default_for_plugin(cls, plugin: str) -> Identity:
        ...

    def plugin(self) -> str:
        ...


class IdentityPluginV1:
    def __new__(cls, plugin_name: str, identities: Sequence[Identity], callbacks: Callbacks) -> Self:
        ...



import unittest

from pyrage import x25519, IdentityError, RecipientError


class TestIdentity(unittest.TestCase):
    def test_generate(self):
        identity = x25519.Identity.generate()
        self.assertIsInstance(identity, x25519.Identity)
        self.assertTrue(str(identity).startswith("AGE-SECRET-KEY"))

        recipient = identity.to_public()
        self.assertTrue(str(recipient).startswith("age"))

    def test_from_str(self):
        generated = x25519.Identity.generate()
        parsed = x25519.Identity.from_str(str(generated))
        self.assertIsInstance(parsed, x25519.Identity)

    def test_from_str_invalid(self):
        with self.assertRaisesRegex(IdentityError, "invalid Bech32 encoding"):
            x25519.Identity.from_str("BAD-PREFIX")


class TestRecipient(unittest.TestCase):
    def test_from_str(self):
        recipient = x25519.Recipient.from_str(
            "age1zvkyg2lqzraa2lnjvqej32nkuu0ues2s82hzrye869xeexvn73equnujwj"
        )
        self.assertIsInstance(recipient, x25519.Recipient)
        self.assertEqual(
            str(recipient),
            "age1zvkyg2lqzraa2lnjvqej32nkuu0ues2s82hzrye869xeexvn73equnujwj",
        )

    def test_from_str_invalid(self):
        with self.assertRaisesRegex(RecipientError, "invalid Bech32 encoding"):
            x25519.Recipient.from_str("badprefix")


if __name__ == "__main__":
    unittest.main()



import os
import tempfile
import unittest
from io import BytesIO

import pyrage

from .utils import ssh_keypair


class TestPyrage(unittest.TestCase):
    def test_encrypt_fails_with_no_receipients(self):
        with self.assertRaisesRegex(
            pyrage.EncryptError, "expected at least one recipient"
        ):
            pyrage.encrypt(b"test", [])

    def test_roundtrip(self):
        identity = pyrage.x25519.Identity.generate()
        recipient = identity.to_public()

        encrypted = pyrage.encrypt(b"test", [recipient])
        decrypted = pyrage.decrypt(encrypted, [identity])

        self.assertEqual(b"test", decrypted)

    def test_roundtrip_io_fh(self):
        identity = pyrage.x25519.Identity.generate()
        recipient = identity.to_public()
        with tempfile.TemporaryFile() as unencrypted:
            unencrypted.write(b"test")
            unencrypted.seek(0)
            with tempfile.TemporaryFile() as encrypted:
                pyrage.encrypt_io(unencrypted, encrypted, [recipient])
                encrypted.seek(0)
                with tempfile.TemporaryFile() as decrypted:
                    pyrage.decrypt_io(encrypted, decrypted, [identity])
                    decrypted.seek(0)
                    unencrypted.seek(0)
                    self.assertEqual(unencrypted.read(), decrypted.read())

    def test_roundtrip_io_bytesio(self):
        identity = pyrage.x25519.Identity.generate()
        recipient = identity.to_public()
        unencrypted = BytesIO(b'test')
        encrypted = BytesIO()
        decrypted = BytesIO()
        pyrage.encrypt_io(unencrypted, encrypted, [recipient])
        encrypted.seek(0)
        pyrage.decrypt_io(encrypted, decrypted, [identity])
        decrypted.seek(0)
        unencrypted.seek(0)
        self.assertEqual(unencrypted.read(), decrypted.read())

    def test_roundtrip_io_fail(self):
        identity = pyrage.x25519.Identity.generate()
        recipient = identity.to_public()

        with self.assertRaises(TypeError):
            input = 'test'
            output = BytesIO()
            pyrage.encrypt_io(input, output, [recipient])

        with self.assertRaises(TypeError):
            input = BytesIO()
            output = 'test'
            pyrage.encrypt_io(input, output, [recipient])

        with self.assertRaises(TypeError):
            input = 'test'
            output = BytesIO()
            pyrage.decrypt_io(input, output, [recipient])

        with self.assertRaises(TypeError):
            input = BytesIO()
            output = 'test'
            pyrage.decrypt_io(input, output, [recipient])


    def test_roundtrip_file(self):
        identity = pyrage.x25519.Identity.generate()
        recipient = identity.to_public()

        with tempfile.TemporaryDirectory() as tempdir:
            unencrypted = os.path.join(tempdir, "unencrypted")
            encrypted = os.path.join(tempdir, "encrypted")
            decrypted = os.path.join(tempdir, "decrypted")

            with open(unencrypted, "wb") as file:
                file.write(b"test")

            pyrage.encrypt_file(unencrypted, encrypted, [recipient])
            pyrage.decrypt_file(encrypted, decrypted, [identity])

            with open(unencrypted, "rb") as file1:
                with open(decrypted, "rb") as file2:
                    self.assertEqual(file1.read(), file2.read())

    def test_decrypt_fails_wrong_recipient(self):
        alice = pyrage.x25519.Identity.generate()
        bob = pyrage.x25519.Identity.generate()

        # alice encrypts to herself
        encrypted = pyrage.encrypt(b"test", [alice.to_public()])

        # bob tries to decrypt and fails
        with self.assertRaisesRegex(pyrage.DecryptError, "No matching keys found"):
            pyrage.decrypt(encrypted, [bob])

        # one key matches, so decryption succeeds
        decrypted = pyrage.decrypt(encrypted, [alice, bob])
        self.assertEqual(b"test", decrypted)

    def test_roundtrip_matrix(self):
        identities = []
        recipients = []

        age_identity = pyrage.x25519.Identity.generate()
        identities.append(age_identity)
        age_recipient = age_identity.to_public()
        recipients.append(age_recipient)

        for filename in ["ed25519", "rsa4096", "rsa2048"]:
            pubkey, privkey = ssh_keypair(filename)
            identities.append(pyrage.ssh.Identity.from_buffer(privkey.encode()))
            recipients.append(pyrage.ssh.Recipient.from_str(pubkey))

        # Encrypt to all recipients, decode using each identity.
        encrypted = pyrage.encrypt(b"test matrix", recipients)
        for identity in identities:
            self.assertEqual(b"test matrix", pyrage.decrypt(encrypted, [identity]))


if __name__ == "__main__":
    unittest.main()



from io import BufferedIOBase
from typing import Sequence, Union

from pyrage import passphrase, plugin, ssh, x25519
from pyrage.plugin import IdentityPluginV1, RecipientPluginV1
from pyrage.ssh import Identity as SSHIdentity
from pyrage.ssh import Recipient as SSHRecipient
from pyrage.x25519 import Identity as X25519Identity
from pyrage.x25519 import Recipient as X25519Recipient

_Identity = Union[SSHIdentity, X25519Identity, IdentityPluginV1]
_Recipient = Union[SSHRecipient, X25519Recipient, RecipientPluginV1]

__all__ = (
    "ssh",
    "x25519",
    "passphrase",
    "plugin",
    "encrypt",
    "encrypt_file",
    "encrypt_io",
    "decrypt",
    "decrypt_file",
    "decrypt_io",
    "RecipientError",
    "IdentityError",
)


class RecipientError(Exception):
    ...

class IdentityError(Exception):
    ...


def encrypt(plaintext: bytes, recipients: Sequence[_Recipient]) -> bytes: ...
def encrypt_file(infile: str, outfile: str, recipients: Sequence[_Recipient]) -> None: ...
def encrypt_io(in_io: BufferedIOBase, out_io: BufferedIOBase, recipients: Sequence[_Recipient]) -> bytes: ...

def decrypt(ciphertext: bytes, identities: Sequence[_Identity]) -> bytes: ...
def decrypt_file(infile: str, outfile: str, identities: Sequence[_Identity]) -> None: ...
def decrypt_io(in_io: BufferedIOBase, out_io: BufferedIOBase, identities: Sequence[_Identity]) -> None: ...
